import * as React from 'react';
import { type ReactElement, type ReactNode } from 'react';
import { type ListControllerProps, type SortPayload, type FilterPayload, type RaRecord } from 'ra-core';
import { type ComponentsOverrides, type SxProps, type Theme } from '@mui/material/styles';
import type { UseQueryOptions } from '@tanstack/react-query';
import type { FieldProps } from './types';
/**
 * A container component that fetches records from another resource specified
 * by an array of *ids* in current record.
 *
 * You must define the fields to be passed to the iterator component as children.
 *
 * @example Display all the products of the current order as datagrid
 * // order = {
 * //   id: 123,
 * //   product_ids: [456, 457, 458],
 * // }
 * <ReferenceArrayField label="Products" reference="products" source="product_ids">
 *     <Datagrid>
 *         <TextField source="id" />
 *         <TextField source="description" />
 *         <NumberField source="price" options={{ style: 'currency', currency: 'USD' }} />
 *         <EditButton />
 *     </Datagrid>
 * </ReferenceArrayField>
 *
 * @example Display all the categories of the current product as a list of chips
 * // product = {
 * //   id: 456,
 * //   category_ids: [11, 22, 33],
 * // }
 * <ReferenceArrayField label="Categories" reference="categories" source="category_ids">
 *     <SingleFieldList>
 *         <ChipField source="name" />
 *     </SingleFieldList>
 * </ReferenceArrayField>
 *
 * By default, restricts the displayed values to 1000. You can extend this limit
 * by setting the `perPage` prop.
 *
 * @example
 * <ReferenceArrayField perPage={10} reference="categories" source="category_ids">
 *    ...
 * </ReferenceArrayField>
 *
 * By default, the field displays the results in the order in which they are referenced
 * (i.e. in the order of the list of ids). You can change this order
 * by setting the `sort` prop (an object with `field` and `order` properties).
 *
 * @example
 * <ReferenceArrayField sort={{ field: 'name', order: 'ASC' }} reference="categories" source="category_ids">
 *    ...
 * </ReferenceArrayField>
 *
 * Also, you can filter the results to display only a subset of values. Use the
 * `filter` prop for that.
 *
 * @example
 * <ReferenceArrayField filter={{ is_published: true }} reference="categories" source="category_ids">
 *    ...
 * </ReferenceArrayField>
 */
export declare const ReferenceArrayField: <RecordType extends RaRecord<import("ra-core").Identifier> = RaRecord<import("ra-core").Identifier>, ReferenceRecordType extends RaRecord<import("ra-core").Identifier> = RaRecord<import("ra-core").Identifier>>(inProps: ReferenceArrayFieldProps<RecordType, ReferenceRecordType>) => React.JSX.Element;
export interface ReferenceArrayFieldProps<RecordType extends RaRecord = RaRecord, ReferenceRecordType extends RaRecord = RaRecord> extends FieldProps<RecordType> {
    children?: ReactNode;
    filter?: FilterPayload;
    page?: number;
    pagination?: ReactElement;
    perPage?: number;
    reference: string;
    sort?: SortPayload;
    sx?: SxProps<Theme>;
    queryOptions?: Omit<UseQueryOptions<ReferenceRecordType[], Error>, 'queryFn' | 'queryKey'>;
}
export interface ReferenceArrayFieldViewProps extends Omit<ReferenceArrayFieldProps, 'resource' | 'page' | 'perPage'>, Omit<ListControllerProps, 'queryOptions'> {
}
export declare const ReferenceArrayFieldView: (props: ReferenceArrayFieldViewProps) => React.JSX.Element;
export declare const ReferenceArrayFieldClasses: {
    progress: string;
};
declare module '@mui/material/styles' {
    interface ComponentNameToClassKey {
        RaReferenceArrayField: 'root' | 'progress';
    }
    interface ComponentsPropsList {
        RaReferenceArrayField: Partial<ReferenceArrayFieldProps>;
    }
    interface Components {
        RaReferenceArrayField?: {
            defaultProps?: ComponentsPropsList['RaReferenceArrayField'];
            styleOverrides?: ComponentsOverrides<Omit<Theme, 'components'>>['RaReferenceArrayField'];
        };
    }
}
//# sourceMappingURL=ReferenceArrayField.d.ts.map