import React, { type ReactElement, type ReactNode } from 'react';
import { type ComponentsOverrides } from '@mui/material/styles';
import { type LinkProps } from 'react-router-dom';
import { type MenuItemProps, type TooltipProps } from '@mui/material';
/**
 * Displays a menu item with a label and an icon - or only the icon with a tooltip when the sidebar is minimized.
 * It also handles the automatic closing of the menu on tap on mobile.
 *
 * @typedef {Object} Props the props you can use
 * @prop {string|Location} to The menu item's target. It is passed to a React Router NavLink component.
 * @prop {string|ReactNode} primaryText The menu content, displayed when the menu isn't minimized. |
 * @prop {ReactNode} leftIcon The menu icon
 *
 * Additional props are passed down to the underling Material UI <MenuItem> component
 * @see https://material-ui.com/api/menu-item/#menuitem-api
 *
 * @example // You can create a custom menu component using the <DashboardMenuItem> and <MenuItemLink> components:
 *
 * // in src/Menu.js
 * import * as React from 'react';
 * import { DashboardMenuItem, MenuItemLink } from 'react-admin';
 * import BookIcon from '@mui/icons-material/Book';
 * import ChatBubbleIcon from '@mui/icons-material/ChatBubble';
 * import PeopleIcon from '@mui/icons-material/People';
 * import LabelIcon from '@mui/icons-material/Label';
 *
 * export const Menu = () => (
 *     <div>
 *         <DashboardMenuItem />
 *         <MenuItemLink to="/posts" primaryText="Posts" leftIcon={<BookIcon />}/>
 *         <MenuItemLink to="/comments" primaryText="Comments" leftIcon={<ChatBubbleIcon />}/>
 *         <MenuItemLink to="/users" primaryText="Users" leftIcon={<PeopleIcon />}/>
 *         <MenuItemLink to="/custom-route" primaryText="Miscellaneous" leftIcon={<LabelIcon />}/>
 *     </div>
 * );
 *
 * // to use this custom menu component, pass it to a custom Layout:
 * // in src/Layout.js
 * import { Layout } from 'react-admin';
 * import { Menu } from './Menu';
 *
 * export const Layout = ({ children }) => (
 *     <Layout menu={Menu}>
 *         {children}
 *     </Layout>
 * );
 *
 * // then, use this layout in the <Admin layout> prop:
 * // in src/App.js
 * import { Layout }  from './Layout';
 *
 * const App = () => (
 *     <Admin layout={Layout} dataProvider={simpleRestProvider('http://path.to.my.api')}>
 *         // ...
 *     </Admin>
 * );
 */
export declare const MenuItemLink: React.ForwardRefExoticComponent<Omit<MenuItemLinkProps, "ref"> & React.RefAttributes<any>>;
export type MenuItemLinkProps = Omit<LinkProps & MenuItemProps<'li'>, 'placeholder' | 'onPointerEnterCapture' | 'onPointerLeaveCapture'> & {
    leftIcon?: ReactElement;
    primaryText?: ReactNode;
    /**
     * @deprecated
     */
    sidebarIsOpen?: boolean;
    tooltipProps?: TooltipProps;
};
export declare const MenuItemLinkClasses: {
    active: string;
    icon: string;
};
declare module '@mui/material/styles' {
    interface ComponentNameToClassKey {
        RaMenuItemLink: 'root' | 'active' | 'icon';
    }
    interface ComponentsPropsList {
        RaMenuItemLink: Partial<MenuItemLinkProps>;
    }
    interface Components {
        RaMenuItemLink?: {
            defaultProps?: ComponentsPropsList['RaMenuItemLink'];
            styleOverrides?: ComponentsOverrides<Omit<Theme, 'components'>>['RaMenuItemLink'];
        };
    }
}
//# sourceMappingURL=MenuItemLink.d.ts.map